class Page < Item
  include Rails.application.routes.url_helpers
  include Impressionist::Impressionable
  include ParserFrontEnd
  
  is_impressionable
  
  field :content, localize: true
  field :app_frontend_url
  # field :theme_id, :type => BSON::ObjectId, :default => nil
  field :category,type: Array, :default => []
  field :tag, type: Array,:default => []
  # field :view_count, :type => Integer, :default => 0
  field :page_title, localize: true
  
  field :frontend_style
  field :frontend_field , :type => Array,:default => []

  field :frontend_field_name , :type => Array,:default => []
  field :frontend_class , :type => Array,:default => []
  field :frontend_sat_to_link , :type => Array,:default => []

  field :frontend_data_count

  belongs_to :theme  
  belongs_to :design
  belongs_to :module_app
  has_many :page_parts, :dependent => :destroy
  has_many :page_contexts, :autosave => true, :dependent => :destroy
  has_many :page_metas, :autosave => true, :dependent => :destroy
  accepts_nested_attributes_for :page_parts, :allow_destroy => true

  before_save :create_parts, if: Proc.new { |page| page.new_record? || page.design_id_changed? }
  before_save :delete_empty_frontend_field, :generate_html

  def generate_parts
    create_parts
    self.save
  end

  def frontend_with_link
    frontend_sat_to_link.map{|d| (d && d.eql?("true")) ? true : false}
  end

  def default_widget_field_sat_to_link?(i)
    case frontend_sat_to_link[i]
      when 'true'
        true
      when 'false'
        false
      else
        nil
    end
  end

  def default_widget_field_name
    return frontend_field_name
  end

  def frontend_field=(settings)
    settings.each_with_index do |setting,index|
      self.frontend_field_name[index] =  setting["field_name"]
      self.frontend_class[index] = setting["class"]
      self.frontend_sat_to_link[index] = setting["sat_to_link"]
    end
  end

  def view_count
    self.impression_count
  end

  protected
  
  def create_parts
    page_design = self.design
    parent = self.parent
    menu_part = parent.page_parts.detect{|page_part| page_part.kind.eql?('public_r_tag') && page_part.public_r_tag.eql?('sub_menu') && page_part.public_r_tag_object_id.eql?(parent.id.to_s)} if parent
    page_design.layout.layout_parts.each do |layout_part|
      current_part = self.page_parts.detect{|page_part| page_part.name.eql?(layout_part.name)}
      current_part = self.page_parts.build(:name => layout_part.name) unless current_part

      if menu_part && current_part.name.eql?(menu_part.name) 
        if current_part.new_record?
          current_part.kind = menu_part.kind
          current_part.public_r_tag = menu_part.public_r_tag
          current_part.public_r_tag_object_id = menu_part.public_r_tag_object_id
          PagePart.without_callback(:save, :after, :update_parent) do
            current_part.save
          end
        else
          PagePart.without_callback(:save, :after, :update_parent) do
            current_part.update_attributes(:kind => menu_part.kind, :public_r_tag => menu_part.public_r_tag, :public_r_tag_object_id => menu_part.public_r_tag_object_id)
          end
        end
      elsif current_part.new_record?
        PagePart.without_callback(:save, :after, :update_parent) do
          current_part.save
        end
      end
    end
  end

  def delete_empty_frontend_field
    if self.frontend_field
        self.frontend_field.reject! { |ff| (ff[0].blank? || ff[1].blank?) }
      # self.frontend_field.each do | ff |
        # if ff[0].blank? || ff[1].blank?
            # self.frontend_field.delete("") if self.frontend_field
            # self.frontend_field.delete(ff)
       # end
      # end
    end
  end

  def generate_html
    self.content_translations = parse_page_noko(self, Site.first)
  end
  
end
